/* Audio-Player mit GStreamer */

#include <gst/gst.h>
#include <glib.h>

/* End-of-Stream Message-Listener */
static gboolean
bus_call(GstBus * bus, GstMessage * msg, gpointer data)
{
	if (GST_MESSAGE_TYPE(msg) == GST_MESSAGE_EOS)
		g_main_loop_quit((GMainLoop *) data);
	return TRUE;
}

/* Signal-Handler fr Demultiplexer */
static void
on_pad_added(GstElement * element, GstPad * pad, gpointer data)
{
	GstPad *sinkpad = gst_element_get_static_pad((GstElement *) data,
												 "sink");

	gst_pad_link(pad, sinkpad);
	gst_object_unref(sinkpad);
}

int
main(int argc, char *argv[])
{
	GMainLoop *loop;
	GstElement *pipeline, *src, *demux, *decoder, *conv, *sink;
	GstBus *bus;

	gst_init(&argc, &argv);

	/* Elemente erzeugen */
	loop = g_main_loop_new(NULL, FALSE);
	pipeline = gst_pipeline_new("audio-player");
	src = gst_element_factory_make("filesrc", "file-source");
	demux = gst_element_factory_make("oggdemux", "ogg-demuxer");
	decoder = gst_element_factory_make("vorbisdec", "vorbis-decoder");
	conv = gst_element_factory_make("audioconvert", "converter");
	sink = gst_element_factory_make("autoaudiosink", "audio-output");

	/* Pipeline vorbereiten */
	g_object_set(G_OBJECT(src), "location", argv[1], NULL);
	bus = gst_pipeline_get_bus(GST_PIPELINE(pipeline));
	gst_bus_add_watch(bus, bus_call, loop);
	gst_object_unref(bus);

	/* Pipeline-Elemente verbinden */
	gst_bin_add_many(GST_BIN(pipeline), src, demux,
					 decoder, conv, sink, NULL);
	gst_element_link(src, demux);
	gst_element_link_many(decoder, conv, sink, NULL);
	g_signal_connect(demux, "pad-added", G_CALLBACK(on_pad_added), decoder);

	/* Start der Wiedergabe */
	gst_element_set_state(pipeline, GST_STATE_PLAYING);
	g_main_loop_run(loop);

	/* Aufrumen */
	gst_element_set_state(pipeline, GST_STATE_NULL);
	gst_object_unref(GST_OBJECT(pipeline));
	exit(0);
}
