/* Gerst eines Video-Konverters mit ffmpeg */

#include <cstdlib>
#include <avcodec.h>
#include <avformat.h>

/* Frame einlesen */
bool
get_next_frame(AVFormatContext * format,
			   AVCodecContext * codec, int stream, AVFrame * frame)
{
	static AVPacket packet;
	static uint8_t *rawData;
	static bool endOfStream = false;
	int bytesRemaining, bytesDecoded;
	static int frameFinished = 0;

	/* Pakete lesen und dekodieren */
	while (!endOfStream && !frameFinished) {
		do {
			if (av_read_packet(format, &packet) < 0)
				endOfStream = true;
		} while (packet.stream_index != stream && !endOfStream);
		bytesRemaining = packet.size;
		rawData = packet.data;
		while (bytesRemaining > 0 && !frameFinished) {
			bytesDecoded =
			   avcodec_decode_video(codec, frame,
									&frameFinished, rawData, bytesRemaining);
			bytesRemaining -= bytesDecoded;
			rawData += bytesDecoded;
		}
	}
	return frameFinished;
}

int
main(int argc, char *argv[])
{
	AVFormatContext *format;
	AVCodecContext *codec;
	AVFrame *frame, *frameRGB;
	uint8_t *buffer;
	int i, stream, numBytes;

	av_register_all();

	/* Datei ffnen und Video-Stream suchen */
	av_open_input_file(&format, argv[1], NULL, 0, NULL);
	av_find_stream_info(format);
	for (i = 0, stream = -1; stream == -1 && i < format->nb_streams; i++)
		if (format->streams[i]->codec->codec_type == CODEC_TYPE_VIDEO)
			stream = i;

	/* passenden Codec finden */
	codec = format->streams[stream]->codec;
	avcodec_open(codec, avcodec_find_decoder(codec->codec_id));

	/* Puffer anlegen */
	frame = avcodec_alloc_frame();
	frameRGB = avcodec_alloc_frame();
	numBytes = avpicture_get_size(PIX_FMT_RGB24, codec->width, codec->height);
	buffer = new uint8_t[numBytes];
	avpicture_fill((AVPicture *) frameRGB, buffer,
				   PIX_FMT_RGB24, codec->width, codec->height);

	/* Frames nach RGB konvertieren */
	while (get_next_frame(format, codec, stream, frame)) {
		img_convert((AVPicture *) frameRGB,
					PIX_FMT_RGB24,
					(AVPicture *) frame,
					codec->pix_fmt, codec->width, codec->height);

		/* Bild verarbeiten/ausgeben (fehlt) */
	}

	/* Aufrumen */
	delete[]buffer;
	av_free(frameRGB);
	av_free(frame);
	avcodec_close(codec);
	av_close_input_file(format);
	exit(0);
}
